/****************************************************************************
** Meta object code from reading C++ file 'gatemodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kosmindoormap-26.08.0/src/map/content/gatemodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'gatemodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KOSMIndoorMap9GateModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KOSMIndoorMap::GateModel::qt_create_metaobjectdata<qt_meta_tag_ZN13KOSMIndoorMap9GateModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KOSMIndoorMap::GateModel",
        "mapDataChanged",
        "",
        "gateIndexChanged",
        "setArrivalGate",
        "name",
        "setDepartureGate",
        "mapData",
        "KOSMIndoorMap::MapData",
        "isEmpty",
        "arrivalGateRow",
        "departureGateRow",
        "Role",
        "CoordinateRole",
        "ElementRole",
        "LevelRole",
        "ArrivalGateRole",
        "DepartureGateRole",
        "HiddenElementRole"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'mapDataChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'gateIndexChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'setArrivalGate'
        QtMocHelpers::MethodData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Method 'setDepartureGate'
        QtMocHelpers::MethodData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'mapData'
        QtMocHelpers::PropertyData<KOSMIndoorMap::MapData>(7, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'isEmpty'
        QtMocHelpers::PropertyData<bool>(9, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'arrivalGateRow'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags, 1),
        // property 'departureGateRow'
        QtMocHelpers::PropertyData<int>(11, QMetaType::Int, QMC::DefaultPropertyFlags, 1),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Role'
        QtMocHelpers::EnumData<enum Role>(12, 12, QMC::EnumFlags{}).add({
            {   13, Role::CoordinateRole },
            {   14, Role::ElementRole },
            {   15, Role::LevelRole },
            {   16, Role::ArrivalGateRole },
            {   17, Role::DepartureGateRole },
            {   18, Role::HiddenElementRole },
        }),
    };
    return QtMocHelpers::metaObjectData<GateModel, qt_meta_tag_ZN13KOSMIndoorMap9GateModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KOSMIndoorMap::GateModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap9GateModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap9GateModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KOSMIndoorMap9GateModelE_t>.metaTypes,
    nullptr
} };

void KOSMIndoorMap::GateModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GateModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->mapDataChanged(); break;
        case 1: _t->gateIndexChanged(); break;
        case 2: _t->setArrivalGate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->setDepartureGate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (GateModel::*)()>(_a, &GateModel::mapDataChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (GateModel::*)()>(_a, &GateModel::gateIndexChanged, 1))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KOSMIndoorMap::MapData >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<KOSMIndoorMap::MapData*>(_v) = _t->mapData(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isEmpty(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->arrivalGateRow(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->departureGateRow(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setMapData(*reinterpret_cast<KOSMIndoorMap::MapData*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KOSMIndoorMap::GateModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KOSMIndoorMap::GateModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap9GateModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int KOSMIndoorMap::GateModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void KOSMIndoorMap::GateModel::mapDataChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KOSMIndoorMap::GateModel::gateIndexChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
