/****************************************************************************
** Meta object code from reading C++ file 'floorlevelchangemodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kosmindoormap-26.08.0/src/map-quick/floorlevelchangemodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'floorlevelchangemodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KOSMIndoorMap21FloorLevelChangeModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KOSMIndoorMap::FloorLevelChangeModel::qt_create_metaobjectdata<qt_meta_tag_ZN13KOSMIndoorMap21FloorLevelChangeModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KOSMIndoorMap::FloorLevelChangeModel",
        "contentChanged",
        "",
        "currentFloorLevel",
        "currentFloorLevelRow",
        "floorLevelModel",
        "KOSMIndoorMap::FloorLevelModel*",
        "element",
        "KOSMIndoorMap::OSMElement",
        "hasSingleLevelChange",
        "destinationLevel",
        "destinationLevelName",
        "hasMultipleLevelChanges",
        "title"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'contentChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'currentFloorLevel'
        QtMocHelpers::PropertyData<int>(3, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'currentFloorLevelRow'
        QtMocHelpers::PropertyData<int>(4, QMetaType::Int, QMC::DefaultPropertyFlags, 0),
        // property 'floorLevelModel'
        QtMocHelpers::PropertyData<KOSMIndoorMap::FloorLevelModel*>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'element'
        QtMocHelpers::PropertyData<KOSMIndoorMap::OSMElement>(7, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'hasSingleLevelChange'
        QtMocHelpers::PropertyData<bool>(9, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'destinationLevel'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags, 0),
        // property 'destinationLevelName'
        QtMocHelpers::PropertyData<QString>(11, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
        // property 'hasMultipleLevelChanges'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'title'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FloorLevelChangeModel, qt_meta_tag_ZN13KOSMIndoorMap21FloorLevelChangeModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KOSMIndoorMap::FloorLevelChangeModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap21FloorLevelChangeModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap21FloorLevelChangeModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KOSMIndoorMap21FloorLevelChangeModelE_t>.metaTypes,
    nullptr
} };

void KOSMIndoorMap::FloorLevelChangeModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FloorLevelChangeModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->contentChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FloorLevelChangeModel::*)()>(_a, &FloorLevelChangeModel::contentChanged, 0))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KOSMIndoorMap::FloorLevelModel* >(); break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KOSMIndoorMap::OSMElement >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->currentFloorLevel(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->currentFloorLevelRow(); break;
        case 2: *reinterpret_cast<KOSMIndoorMap::FloorLevelModel**>(_v) = _t->floorLevelModel(); break;
        case 3: *reinterpret_cast<KOSMIndoorMap::OSMElement*>(_v) = _t->element(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->hasSingleLevelChange(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->destinationLevel(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->destinationLevelName(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->hasMultipleLevelChanges(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->title(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setCurrentFloorLevel(*reinterpret_cast<int*>(_v)); break;
        case 2: _t->setFloorLevelModel(*reinterpret_cast<KOSMIndoorMap::FloorLevelModel**>(_v)); break;
        case 3: _t->setElement(*reinterpret_cast<KOSMIndoorMap::OSMElement*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KOSMIndoorMap::FloorLevelChangeModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KOSMIndoorMap::FloorLevelChangeModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap21FloorLevelChangeModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int KOSMIndoorMap::FloorLevelChangeModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void KOSMIndoorMap::FloorLevelChangeModel::contentChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
