/****************************************************************************
** Meta object code from reading C++ file 'mappointerevent.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kosmindoormap-26.08.0/src/map-quick/mappointerevent.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mappointerevent.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KOSMIndoorMap15MapPointerEventE_t {};
} // unnamed namespace

template <> constexpr inline auto KOSMIndoorMap::MapPointerEvent::qt_create_metaobjectdata<qt_meta_tag_ZN13KOSMIndoorMap15MapPointerEventE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KOSMIndoorMap::MapPointerEvent",
        "QML.Element",
        "auto",
        "mapPointerEvent",
        "QML.Creatable",
        "true",
        "QML.CreationMethod",
        "structured",
        "element",
        "KOSMIndoorMap::OSMElement",
        "geoPosition",
        "QPointF",
        "screenPosition",
        "button",
        "Qt::MouseButton",
        "modifiers"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'element'
        QtMocHelpers::PropertyData<KOSMIndoorMap::OSMElement>(8, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'geoPosition'
        QtMocHelpers::PropertyData<QPointF>(10, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'screenPosition'
        QtMocHelpers::PropertyData<QPointF>(12, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'button'
        QtMocHelpers::PropertyData<Qt::MouseButton>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'modifiers'
        QtMocHelpers::PropertyData<int>(15, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    1,    3 },
            {    4,    5 },
            {    6,    7 },
    });
    return QtMocHelpers::metaObjectData<MapPointerEvent, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject KOSMIndoorMap::MapPointerEvent::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap15MapPointerEventE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap15MapPointerEventE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KOSMIndoorMap15MapPointerEventE_t>.metaTypes,
    nullptr
} };

void KOSMIndoorMap::MapPointerEvent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<MapPointerEvent *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KOSMIndoorMap::OSMElement >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<KOSMIndoorMap::OSMElement*>(_v) = _t->m_element; break;
        case 1: *reinterpret_cast<QPointF*>(_v) = _t->m_geoPos; break;
        case 2: *reinterpret_cast<QPointF*>(_v) = _t->m_screenPos; break;
        case 3: *reinterpret_cast<Qt::MouseButton*>(_v) = _t->m_button; break;
        case 4: *reinterpret_cast<int*>(_v) = _t->m_modifiers; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->m_element, *reinterpret_cast<KOSMIndoorMap::OSMElement*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->m_geoPos, *reinterpret_cast<QPointF*>(_v)); break;
        case 2: QtMocHelpers::setProperty(_t->m_screenPos, *reinterpret_cast<QPointF*>(_v)); break;
        case 3: QtMocHelpers::setProperty(_t->m_button, *reinterpret_cast<Qt::MouseButton*>(_v)); break;
        case 4: QtMocHelpers::setProperty(_t->m_modifiers, *reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
