/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<mappointerevent.h>)
#  include <mappointerevent.h>
#endif
#if __has_include(<types.h>)
#  include <types.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_org_kde_kosmindoormap()
{
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<AmenityModelForeign>("org.kde.kosmindoormap", 1);
    qmlRegisterEnum<KOSMIndoorMap::AmenityModel::Role>("KOSMIndoorMap::AmenityModel::Role");
    qmlRegisterEnum<KOSMIndoorMap::AmenityModel::Group>("KOSMIndoorMap::AmenityModel::Group");
    qmlRegisterTypesAndRevisions<AmenitySortFilterProxyModelForeign>("org.kde.kosmindoormap", 1);
    qmlRegisterTypesAndRevisions<EquipmentModelForeign>("org.kde.kosmindoormap", 1);
    qmlRegisterTypesAndRevisions<FloorLevelChangeModelForeign>("org.kde.kosmindoormap", 1);
    qmlRegisterTypesAndRevisions<FloorLevelModelForeign>("org.kde.kosmindoormap", 1);
    qmlRegisterTypesAndRevisions<GateModelForeign>("org.kde.kosmindoormap", 1);
    qmlRegisterEnum<KOSMIndoorMap::GateModel::Role>("KOSMIndoorMap::GateModel::Role");
    QMetaType::fromType<KOSMIndoorMap::AbstractOverlaySource *>().id();
    qmlRegisterTypesAndRevisions<KOSMIndoorMap::MapPointerEvent>("org.kde.kosmindoormap", 1);
    qmlRegisterTypesAndRevisions<MapDataForeign>("org.kde.kosmindoormap", 1);
    qmlRegisterTypesAndRevisions<MapItemForeign>("org.kde.kosmindoormap", 1);
    qmlRegisterTypesAndRevisions<MapLoaderForeign>("org.kde.kosmindoormap", 1);
    qmlRegisterTypesAndRevisions<OSMElementForeign>("org.kde.kosmindoormap", 1);
    qmlRegisterTypesAndRevisions<OSMElementInformationModelForeign>("org.kde.kosmindoormap", 1);
    qmlRegisterEnum<KOSMIndoorMap::OSMElementInformationModel::KeyCategory>("KOSMIndoorMap::OSMElementInformationModel::KeyCategory");
    qmlRegisterEnum<KOSMIndoorMap::OSMElementInformationModel::Key>("KOSMIndoorMap::OSMElementInformationModel::Key");
    qmlRegisterEnum<KOSMIndoorMap::OSMElementInformationModel::Type>("KOSMIndoorMap::OSMElementInformationModel::Type");
    QMetaType::fromType<PlatformDerived>().id();
    qmlRegisterNamespaceAndRevisions(&PlatformDerived::staticMetaObject, "org.kde.kosmindoormap", 1, nullptr, &PlatformDerivedForeign::staticMetaObject, nullptr);
    qmlRegisterTypesAndRevisions<PlatformForeign>("org.kde.kosmindoormap", 1);
    qmlRegisterEnum<KOSMIndoorMap::Platform::Mode>("KOSMIndoorMap::Platform::Mode");
    qmlRegisterTypesAndRevisions<PlatformModelForeign>("org.kde.kosmindoormap", 1);
    qmlRegisterEnum<KOSMIndoorMap::PlatformModel::Role>("KOSMIndoorMap::PlatformModel::Role");
    QMetaType::fromType<QAbstractItemModel *>().id();
    qmlRegisterEnum<QAbstractItemModel::LayoutChangeHint>("QAbstractItemModel::LayoutChangeHint");
    qmlRegisterEnum<QAbstractItemModel::CheckIndexOption>("QAbstractItemModel::CheckIndexOption");
    QMetaType::fromType<QAbstractListModel *>().id();
    QMetaType::fromType<QAbstractProxyModel *>().id();
    QMetaType::fromType<QSortFilterProxyModel *>().id();
    qmlRegisterTypesAndRevisions<RoomModelForeign>("org.kde.kosmindoormap", 1);
    qmlRegisterEnum<KOSMIndoorMap::RoomModel::Role>("KOSMIndoorMap::RoomModel::Role");
    qmlRegisterTypesAndRevisions<RoomSortFilterProxyModelForeign>("org.kde.kosmindoormap", 1);
    qmlRegisterTypesAndRevisions<ViewForeign>("org.kde.kosmindoormap", 1);
    QT_WARNING_POP
    qmlRegisterModule("org.kde.kosmindoormap", 1, 0);
}

static const QQmlModuleRegistration orgkdekosmindoormapRegistration("org.kde.kosmindoormap", qml_register_types_org_kde_kosmindoormap);
