// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "render-logging.h"

 namespace KOSMIndoorMap {
Q_LOGGING_CATEGORY(RenderLog, "org.kde.kosmindoormap.renderer", QtInfoMsg)
} 
