/****************************************************************************
** Meta object code from reading C++ file 'platform.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kosmindoormap-26.08.0/src/map/content/platform.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'platform.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KOSMIndoorMap8PlatformE_t {};
} // unnamed namespace

template <> constexpr inline auto KOSMIndoorMap::Platform::qt_create_metaobjectdata<qt_meta_tag_ZN13KOSMIndoorMap8PlatformE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KOSMIndoorMap::Platform",
        "name",
        "mode",
        "Mode",
        "ifopt",
        "level",
        "hasLevel",
        "position",
        "QPointF",
        "lines",
        "Unknown",
        "Rail",
        "LightRail",
        "Subway",
        "Tram",
        "Monorail",
        "Bus"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'name'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'mode'
        QtMocHelpers::PropertyData<enum Mode>(2, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'ifopt'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'level'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'hasLevel'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'position'
        QtMocHelpers::PropertyData<QPointF>(7, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'lines'
        QtMocHelpers::PropertyData<QStringList>(9, QMetaType::QStringList, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Mode'
        QtMocHelpers::EnumData<enum Mode>(3, 3, QMC::EnumFlags{}).add({
            {   10, Mode::Unknown },
            {   11, Mode::Rail },
            {   12, Mode::LightRail },
            {   13, Mode::Subway },
            {   14, Mode::Tram },
            {   15, Mode::Monorail },
            {   16, Mode::Bus },
        }),
    };
    return QtMocHelpers::metaObjectData<Platform, qt_meta_tag_ZN13KOSMIndoorMap8PlatformE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KOSMIndoorMap::Platform::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap8PlatformE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap8PlatformE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KOSMIndoorMap8PlatformE_t>.metaTypes,
    nullptr
} };

void KOSMIndoorMap::Platform::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Platform *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 1: *reinterpret_cast<enum Mode*>(_v) = _t->mode(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->ifopt(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->level(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->hasLevel(); break;
        case 5: *reinterpret_cast<QPointF*>(_v) = _t->positionPoint(); break;
        case 6: *reinterpret_cast<QStringList*>(_v) = _t->lines(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setMode(*reinterpret_cast<enum Mode*>(_v)); break;
        case 2: _t->setIfopt(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
