/****************************************************************************
** Meta object code from reading C++ file 'routingcontroller.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kosmindoormap-26.08.0/src/routing-quick/routingcontroller.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'routingcontroller.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17KOSMIndoorRouting17RoutingControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto KOSMIndoorRouting::RoutingController::qt_create_metaobjectdata<qt_meta_tag_ZN17KOSMIndoorRouting17RoutingControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KOSMIndoorRouting::RoutingController",
        "QML.Element",
        "auto",
        "progressChanged",
        "",
        "mapDataChanged",
        "elevatorModelChanged",
        "profileChanged",
        "searchRoute",
        "setStartPosition",
        "lat",
        "lon",
        "floorLevel",
        "setEndPosition",
        "available",
        "inProgress",
        "mapData",
        "KOSMIndoorMap::MapData",
        "elevatorModel",
        "KOSMIndoorMap::AbstractOverlaySource*",
        "routeOverlay",
        "profile",
        "KOSMIndoorRouting::RoutingProfile"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'progressChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'mapDataChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'elevatorModelChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'profileChanged'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'searchRoute'
        QtMocHelpers::SlotData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'setStartPosition'
        QtMocHelpers::MethodData<void(double, double, int)>(9, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 }, { QMetaType::Double, 11 }, { QMetaType::Int, 12 },
        }}),
        // Method 'setEndPosition'
        QtMocHelpers::MethodData<void(double, double, int)>(13, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 }, { QMetaType::Double, 11 }, { QMetaType::Int, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'available'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'inProgress'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'mapData'
        QtMocHelpers::PropertyData<KOSMIndoorMap::MapData>(16, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'elevatorModel'
        QtMocHelpers::PropertyData<KOSMIndoorMap::AbstractOverlaySource*>(18, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 2),
        // property 'routeOverlay'
        QtMocHelpers::PropertyData<KOSMIndoorMap::AbstractOverlaySource*>(20, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'profile'
        QtMocHelpers::PropertyData<KOSMIndoorRouting::RoutingProfile>(21, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<RoutingController, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject KOSMIndoorRouting::RoutingController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KOSMIndoorRouting17RoutingControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KOSMIndoorRouting17RoutingControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17KOSMIndoorRouting17RoutingControllerE_t>.metaTypes,
    nullptr
} };

void KOSMIndoorRouting::RoutingController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RoutingController *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->progressChanged(); break;
        case 1: _t->mapDataChanged(); break;
        case 2: _t->elevatorModelChanged(); break;
        case 3: _t->profileChanged(); break;
        case 4: _t->searchRoute(); break;
        case 5: _t->setStartPosition((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 6: _t->setEndPosition((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RoutingController::*)()>(_a, &RoutingController::progressChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (RoutingController::*)()>(_a, &RoutingController::mapDataChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (RoutingController::*)()>(_a, &RoutingController::elevatorModelChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (RoutingController::*)()>(_a, &RoutingController::profileChanged, 3))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KOSMIndoorMap::AbstractOverlaySource* >(); break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KOSMIndoorMap::MapData >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->routingAvailable(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->routingInProgress(); break;
        case 2: *reinterpret_cast<KOSMIndoorMap::MapData*>(_v) = _t->m_mapData; break;
        case 3: *reinterpret_cast<KOSMIndoorMap::AbstractOverlaySource**>(_v) = _t->m_elevatorModel; break;
        case 4: *reinterpret_cast<KOSMIndoorMap::AbstractOverlaySource**>(_v) = _t->routeOverlay(); break;
        case 5: *reinterpret_cast<KOSMIndoorRouting::RoutingProfile*>(_v) = _t->m_routingProfile; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setMapData(*reinterpret_cast<KOSMIndoorMap::MapData*>(_v)); break;
        case 3:
            if (QtMocHelpers::setProperty(_t->m_elevatorModel, *reinterpret_cast<KOSMIndoorMap::AbstractOverlaySource**>(_v)))
                Q_EMIT _t->elevatorModelChanged();
            break;
        case 5: _t->setProfile(*reinterpret_cast<KOSMIndoorRouting::RoutingProfile*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KOSMIndoorRouting::RoutingController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KOSMIndoorRouting::RoutingController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KOSMIndoorRouting17RoutingControllerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KOSMIndoorRouting::RoutingController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void KOSMIndoorRouting::RoutingController::progressChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KOSMIndoorRouting::RoutingController::mapDataChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KOSMIndoorRouting::RoutingController::elevatorModelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KOSMIndoorRouting::RoutingController::profileChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
