/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<routingarea.h>)
#  include <routingarea.h>
#endif
#if __has_include(<routingcontroller.h>)
#  include <routingcontroller.h>
#endif
#if __has_include(<types.h>)
#  include <types.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_org_kde_kosmindoorrouting()
{
    qmlRegisterModule("org.kde.kosmindoorrouting", 254, 0);
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    {
        Q_CONSTINIT static auto metaType = QQmlPrivate::metaTypeForNamespace(
            [](const QtPrivate::QMetaTypeInterface *) {return &AreaFlag::staticMetaObject;},
            "AreaFlag");
        QMetaType(&metaType).id();
    }
    qmlRegisterNamespaceAndRevisions(&AreaFlag::staticMetaObject, "org.kde.kosmindoorrouting", 254, nullptr, &AreaFlag::staticMetaObject, nullptr);
    qmlRegisterEnum<AreaFlag::AreaFlag>("AreaFlag::AreaFlag");
    {
        Q_CONSTINIT static auto metaType = QQmlPrivate::metaTypeForNamespace(
            [](const QtPrivate::QMetaTypeInterface *) {return &AreaType::staticMetaObject;},
            "AreaType");
        QMetaType(&metaType).id();
    }
    qmlRegisterNamespaceAndRevisions(&AreaType::staticMetaObject, "org.kde.kosmindoorrouting", 254, nullptr, &AreaType::staticMetaObject, nullptr);
    qmlRegisterEnum<AreaType::AreaType>("AreaType::AreaType");
    qmlRegisterTypesAndRevisions<KOSMIndoorRouting::RoutingController>("org.kde.kosmindoorrouting", 254);
    qmlRegisterTypesAndRevisions<RoutingProfileForeign>("org.kde.kosmindoorrouting", 254);
    QT_WARNING_POP
    qmlRegisterModule("org.kde.kosmindoorrouting", 254, 254);
}

static const QQmlModuleRegistration orgkdekosmindoorroutingRegistration("org.kde.kosmindoorrouting", qml_register_types_org_kde_kosmindoorrouting);
